function [status_SeisSound] = SeisSound_main(fname,sta_favs,chan,net,movie_height,basedir)
%###################################################################
%
% Name: SeisSound_main.m - a script to generate SeisSound movies that contain both audio and video 
%
% Copyright (C) 2012  
%   Debi Kilb at Scripps Institution of Oceanography in collaboration with 
%   Product Team, IRIS Data Management Center. 
%
%    This is a free software; you can redistribute it and/or modify
%    it under the terms of the GNU Lesser General Public License as
%    published by the Free Software Foundation; either version 3 of the
%    License, or (at your option) any later version.
%    
%    This software is distributed in the hope that it will be useful, but
%    WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%    Lesser General Public License (GNU-LGPL) for more details.  The
%    GNU-LGPL and further information can be found here:
%    http://www.gnu.org/
%
%    You should have received a copy of the GNU Affero General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Description:
%
%    For each station/chan/seismogram triplet of interest, this script creates a movie of
%    a series of images of the seismogram, filtered seismogram and spectragram
%    and include an associated sound track of the seismogram.  
%    The movie is placed in a 'Movies' directory and there is also an option to 
%    save separate sound and image files in directories 'Sound' and 'Images',
%    respectively.
%
% Input:
%
%    This function expects:
%       1) fname: name of file with earthquake information (see format below)
%       2) sta_favs: list of stations or station to process
%       3) chan: seismic channel to process
%       4) net: seismic network
%       5) movie_height: height (in pixels) of the final movie
%
%    Format of fname content: 
%       - File that contains parameter information for earthquake to be processed
%       - Parameters are separated with a "|" marker.
%       - Explaination of parameters and a sample record are shown below
%
% EVENT_ID|YYYY|MM|DD|HH|MM|SS|LAT|LON|DEPTH|CONTRIBUTOR|REGION_ID|REGION_CODE|REGION_NAME|MAG_TYPE,MAG,MAG_AUTH|MAG_TYPE,MAG,MAG_AUTH|....
%    Example:
% 3331341|2011|12|14|05|04|59|-7.561|146.804|140.9|NEIC PDE-Q|16|207|EASTERN NEW GUINEA REG., P.N.G.|NEIC PDE|NEIC|MW,7.1,GCMT|MW,7.1,UCMT|MB,6.6,NEIC
%
% History:
%
%    2012-07-19: prepared for public release by IRIS DMC Product Team
%    2012-05-01: Revised for creation of data products at IRIS by Debi Kilb (Scripps)
%    2011 : Zhigang Peng (Georga Tech), Original version 
%
% Files and Paths:
%  
%    Requires matTaup.jar and IRIS-WS-?.?.jar, both of which are included in this release 
%
format compact

%
%  Add paths 
%
setenv('LD_LIBRARY_PATH',['/usr/lib:/usr/lib64:' getenv('LD_LIBRARY_PATH')]);

%
% Define flags to indicate what data output products are requested
%
iflag_sound = 1;  % set to 1 to keep a copy of the wav sound file

iflag_movie = 1;  % set to 1 to create a movie that includes sound and video
%disp('no movie!!!')
%iflag_movie = 0;  % set to 1 to create a movie that includes sound and video

iflag_image = 1;  % set to 1 to keep a sample thumbnail, 0 to not keep a thumbnail image
iflag_phases = 1; % set to 1 to add phase information, 0 to not add phase info

%iflag_vco = 0;    % set to 1 to add voltage controlled oscillator feature, 0 otherwise; requires iflag_movie = 1
iflag_vco = 1;   % set to 1 to add voltage controlled oscillator feature, 0 otherwise; requires iflag_movie = 1

iflag_stereo = 1; % set 1 to have stereo audio, 0 for mono audio
iflag_largescale = 0; % set to 1 to also generate a large scale movie that is 800x900
debug = 0;        % set to 1 for verbose information

%
%  Deinfe static parameters
%
%filt_low = 2;
%filt_high = 8;
filt_low = 0.5;
filt_high = -inf;

filt_low_sound = 0.5; % 
filt_high_sound = -inf;

%filt_low_sound_vco = 8;
%filt_high_sound_vco = 20;
filt_low_sound_vco = -inf;
filt_high_sound_vco = 0.5;

%filt_low_spectra = 0.001; % do not use, saturates the color so one can not see the important signals
%filt_high_spectra = -inf;
filt_low_spectra = 0.5;
filt_high_spectra = -inf;

%speed_factor = 750;
speed_factor = 500;

ColorBar_Upper_Limit = 0;
ColorBar_Lower_Limit=-80;
FramesPerSecond = -9999;
units=' ';
dtype =' ';
status_SeisSound = 1;

pSlowness  = 7.2; % P slowness (s/degree);  a P slowness of ~ 7.2 s/deg (should be conservative out to 125 deg)
rSlowness  = 28.5;  % Rayleigh slowness (s/degree); originally 29.835 but using slowness of 28.5 to provides a better phase display
%iSlowness  = 111.195 / 0.29; % Infrasound slowness (s/degree)

phaseList  = 'P,Pn,Pg,Pdiff,pPdiff,sPdiff,PcP,PP,PP,PS,PKP,SKS,S,Sn,Sg,Sdiff,pSdiff,sSdiff,SP,ScS,SS'; %list of requested seismic phases
phaseSpacing = 200;        % number of seconds between two consecutive phases to avoid over plots

%
%  Loop over all stations of interest
%
for ista = 1:size(sta_favs,1)
   sta = strtrim(sta_favs(ista,:));

   if (debug)
      disp(['Now processing ',fname,'.']);
   end
   %fid=fopen(fname, 'rb');
   [fid,message]=fopen(fname, 'rb');
   if fid<0
      disp(['WARNING[main]: ',message,' ',fname]);
      disp('WARNING[main]: Not able to create requested movie, now exiting')
      status_SeisSound = 0;
      break
   end
%
% Read in data and define metadata
%
   irec = 1;
   while 1
       tline = fgetl(fid);
       if ~ischar(tline), break, end
       if (debug)
          disp(['DEBUG[main]: ',tline])
       end
      [ibreak] = strfind(tline,'|');
      iat1 = 1;
      for i=1:length(ibreak)
          iat2 = ibreak(i)-1;
          var = tline(iat1:iat2);
          vtemp = ['var',num2str(i)];
          eval([vtemp,'=tline(iat1:iat2);']);
          iat1 = iat2+2; % need to add two to avoid the separater '|' character
      end
      iat2 = length(tline);  % now to log the last parameter ...
      var = tline(iat1:iat2);
      vtemp = ['var',num2str(length(ibreak)+1)];
      eval([vtemp,'=tline(iat1:iat2);']);
% 
%  Assume first magnitude field is the reference magnitude
%
      if debug
         disp('WARNING[main]: Assuming first magnitude listed is the reference magnitude, tossing the others.')
      end
      magstr = var17(1:6);
      %magstr(3) = '=';
      metadata(irec).iris_id=var1;
      metadata(irec).yyyy=var2;
      metadata(irec).mm=var3;
      metadata(irec).dd=var4;
      metadata(irec).hh=var5;
      metadata(irec).mn=var6;
      metadata(irec).sec=var7;
      metadata(irec).lat=var8;
      metadata(irec).lon=var9;
      metadata(irec).depth=var10;
      metadata(irec).contrib=var11;
      metadata(irec).region_id=var12; 
      metadata(irec).region_code=var13; 
      metadata(irec).region_name=var14;
      metadata(irec).mag_cat=var15;
      metadata(irec).mag_auth=var16;
      metadata(irec).magstr=magstr;
%  
%   Compute travel time to station of interst
%
      num_yyyy = str2num(metadata(irec).yyyy);
      num_mm = str2num(metadata(irec).mm);
      num_dd = str2num(metadata(irec).dd);
      num_hh = str2num(metadata(irec).hh);
      num_mn = str2num(metadata(irec).mn);
      num_sec = str2num(metadata(irec).sec);

      metadata(irec).eq_yy = num_yyyy;
      metadata(irec).eq_mm = num_mm;
      metadata(irec).eq_dd = num_dd;
      metadata(irec).eq_hh = num_hh;
      metadata(irec).eq_mn = num_mn;
      metadata(irec).eq_sc = num_sec;

      str_time_eq = datestr(datenum(num_yyyy,num_mm,num_dd,num_hh,num_mn,num_sec),'yyyy-mm-dd HH:MM:SS');
      try
         if (debug)
            disp(['DEBUG[main]: Processing station ',sta]);
         end
         if length(net)>0
            s = irisFetch.Stations('STATION',net,sta,[],chan,'EndAfter',str_time_eq)
         else
            s = irisFetch.Stations('STATION',[],sta,[],chan,'EndAfter',str_time_eq)
         end
         %process(s) % what is this, was in sample script from manoch

      catch er
          nodata = 'No data found for';
          if length(er.message)>=89
             if strcmp(er.message(73:89),nodata)
                disp('ERROR[main]: No data found.')
             end
          else
             disp('ERROR[main]: Trouble retrieving data')
          end
         
          disp('ERROR[main]: Not able to create requested movie, now exiting')
          status_SeisSound = 0;
          break
      end % end try
      if length(s)>1
          disp(['WARNING[main]: Station not unique, station ',sta,' belongs to multiple networks'])
          netfav = ['IU';'II'];
          for in = 1:size(netfav,1)
              for k=1:length(s)
                 ismatch = strmatch(netfav(in,:), s(k).NetworkCode, 'exact');
                 if ismatch==1
                    s = s(k);
                    disp(['WARNING[main]: Assigning network code: ',s.NetworkCode]);
                    net = s.NetworkCode;
                    break
                 end % ismatch loop
              end % k loop
          end % in loop
          if length(s)>1
              disp('WARNING[main]: Station belongs to multiple networks, none in favorite list.')
              disp('WARNING[main]:     None of these networks are in our favorite network list.')
              disp('WARNING[main]:     Randomly assigning first network listed.')
              s = s(1);
              disp(['Assigning network code: ',s.Code]);
              net = s.Code;
          end
      end
      sta_lon = s.Longitude;
      sta_lat = s.Latitude;

      eq_lat = str2num(metadata(irec).lat);
      eq_lon = str2num(metadata(irec).lon);
      eq_dep_km =  str2num(metadata(irec).depth);
%
%  Obtain phase arrival times to help select time window of interest
%
      eventYear = str2num(metadata(irec).yyyy);
      eventMonth = str2num(metadata(irec).mm);
      eventDay = str2num(metadata(irec).dd);
      eventHour = str2num(metadata(irec).hh);
      eventMinute = str2num(metadata(irec).mn);
      eventSecond = str2num(metadata(irec).sec);
      eventLat = str2num(metadata(irec).lat);
      eventLon = str2num(metadata(irec).lon);
      eventDepth = str2num(metadata(irec).depth);
%
      eventJday = jday(eventYear, eventMonth, eventDay); % get julian day
      eventEpoch = epoch(eventYear,eventJday,eventHour,eventMinute,eventSecond); %now, translate all events times into epochal
%
%  Compute distance between the earthquake and the station
%
      distgc = distance(eventLat,eventLon,sta_lat,sta_lon);
      [del az] = stadis2(eventLat,eventLon,sta_lat,sta_lon);
      if (debug)
         disp(['The distance between the earthquake and station is: ',num2str(distgc),' deg']);
      end
      metadata(irec).distgc = distgc;
      metadata(irec).eventEpoch = eventEpoch;
%
%   Compute phase arrival times
%
      if(iflag_phases)
         if(debug)
           fprintf('[INFO] eventDepth=%f,phaseList=%s,[sta_lat=%f sta_lon=%f],[eventLat=%f,eventLon=%f]\n',eventDepth,phaseList,sta_lat,sta_lon,eventLat,eventLon);
         end;
         phaseTime = taupTime('iasp91',eventDepth,phaseList,'sta',[sta_lat sta_lon],'evt',[eventLat,eventLon]);
         if(debug)
            for i=1:length(phaseTime)
               fprintf('[INFO] phase: %s time: %f',phaseTime(i).phaseName, phaseTime(i).time);
            end;
         end;
      end;
      metadata(irec).pSlowness=pSlowness;
      metadata(irec).rSlowness=rSlowness;
      %metadata(irec).iSlowness=iSlowness;
      metadata(irec).phaseSpacing=phaseSpacing;
%
%  Define time window of interest based on phase arrivals.
%  Calculate the start and end times of the time window of interest,
%  assign end of time window to be 1800 seconds after R2. 
%
      tt_seconds = min([phaseTime.time]);
      r2Time = ((360 - distgc) * rSlowness);
      %tshift = 0.2*(max([phaseTime.time]) - min([phaseTime.time]));
      tshift = 0.12*(r2Time - tt_seconds); % Define a tshift time that has a length 12% of time window w/phases
      twin_seconds = r2Time + 1800;
      t1 = tt_seconds-tshift;% Define start time to be tshift before the first phase arrival
      t2 = tt_seconds+twin_seconds;
%
      str_time_eq_check = [metadata(irec).yyyy,'-', metadata(irec).mm,'-',metadata(irec).dd,...
                  ' ',metadata(irec).hh,':',metadata(irec).mn,':',metadata(irec).sec];

      %str_time_eq_check = datestr(datenum(2011,12,24,05,04,59),'yyyy-mm-dd HH:MM:SS')
      %str_time_beginit_check = datestr(datenum(2011,12,24,05,04,59+t1),'yyyy-mm-dd HH:MM:SS')
      %str_time_endit_check = datestr(datenum(2011,12,24,05,04,59+t2),'yyyy-mm-dd HH:MM:SS')

      str_time_eq = datestr(datenum(num_yyyy,num_mm,num_dd,num_hh,num_mn,num_sec),'yyyy-mm-dd HH:MM:SS');
      str_time_beginit = datestr(datenum(num_yyyy,num_mm,num_dd,num_hh,num_mn,num_sec+t1),'yyyy-mm-dd HH:MM:SS');
      str_time_endit = datestr(datenum(num_yyyy,num_mm,num_dd,num_hh,num_mn,num_sec+t2),'yyyy-mm-dd HH:MM:SS');

      metadata(irec).str_time_eq = str_time_eq;
      metadata(irec).str_time_beginit = str_time_beginit;
      metadata(irec).str_time_endit = str_time_endit;
      metadata(irec).movie_height = movie_height;
%
%  Save static paramters to metadata structure 
%
      metadata(irec).filt_low = filt_low;
      metadata(irec).filt_high = filt_high;
      metadata(irec).filt_low_sound = filt_low_sound;
      metadata(irec).filt_high_sound = filt_high_sound;
      metadata(irec).filt_low_sound_vco = filt_low_sound_vco;
      metadata(irec).filt_high_sound_vco = filt_high_sound_vco;
      metadata(irec).filt_low_spectra = filt_low_spectra;
      metadata(irec).filt_high_spectra = filt_high_spectra;
      metadata(irec).units = units;
      metadata(irec).dtype = dtype;
      metadata(irec).speed_factor = speed_factor;
      metadata(irec).ColorBar_Upper_Limit = ColorBar_Upper_Limit;
      metadata(irec).ColorBar_Lower_Limit= ColorBar_Lower_Limit;
%
%   Save flags to metadata structure for use in subroutines
%
      metadata(irec).iflag_sound = iflag_sound;
      metadata(irec).iflag_movie = iflag_movie;
      metadata(irec).iflag_image = iflag_image;
      metadata(irec).iflag_phases = iflag_phases;
      metadata(irec).iflag_vco = iflag_vco;
      metadata(irec).iflag_stereo = iflag_stereo;
      metadata(irec).iflag_largescale = iflag_largescale; 
      metadata(irec).debug = debug;

      data = irisFetch.Traces(net,sta,'*',chan,str_time_beginit,str_time_endit);
      %data = irisFetch.Traces('??',sta,'*',chan,str_time_beginit,str_time_endit);
      if length(data)>1
         disp('WARNING[main]: Data returned has more than one seismogram, selecting one with the smallest location tag value')
         for jj=1:length(data)
             if isempty(data(jj).location)
                 disp('WARNING[main]: No location code')
                 loc(jj) = 2; % Not sure, what to do if no location code found?
             else
                loc(jj) = str2num(data(jj).location);
                if length(loc(jj))==0
                  loc(jj) = -1
               end
            end
         end
         [iitemp,jtemp]=min(loc);
         disp(['WARNING[main]: Data redifined to seismic trace ',num2str(jtemp),' out of ',num2str(length(data))]);
         data = data(jtemp);
         if length(data)==1
            if debug
               disp(['DEBUG[main]: Data reassignment successful']);
            end
         end
      end
      if length(data.data)==0
         disp('ERROR[main]: No data available')
         disp('ERROR[main]: Not able to create requested movie, now exiting')
         status_SeisSound = 0;
         break
      end
      temp = irisFetch.Stations('Response',data.network,sta,[],chan);

      if length(temp)>1
         temp.Code
         disp('ERROR[main]: Data reduction failed')
         status_SeisSound = 0;
         break
      end   

      sfull = irisFetch.Channels('Response',data.network,sta,[],chan)
      site = sfull(1).Site;
      metadata(irec).site=site.Name;

      metadata(irec).sampleRate = data.sampleRate;
      metadata(irec).sensitivityUnits = data.sensitivityUnits;
      metadata(irec).azimuth = az;
      metadata(irec).distgc = distgc;
      metadata(irec).instrument = data.instrument;
      metadata(irec).sampleCount = data.sampleCount;

      metadata(irec).dt = 1/data.sampleRate;
      metadata(irec).nf = data.sampleRate/2;
      metadata(irec).ff_max = data.sampleRate/2;

      metadata(irec).filt_low =  filt_low;
      metadata(irec).filt_high = filt_high;
      metadata(irec).speed_factor = speed_factor;
      metadata(irec).ColorBar_Upper_Limit = ColorBar_Upper_Limit;
      metadata(irec).ColorBar_Lower_Limit = ColorBar_Lower_Limit;
      metadata(irec).FramesPerSecond = FramesPerSecond;

      if length(data.data)==0
          disp('ERROR[main]: Retrieved data file is empty, now exiting')
          disp('ERROR[main]: Not able to create requested movie, now exiting')
          status_SeisSound = 0;
          break
      end
%
%  Use the metadata.sensitivityUnits variable to determine the data type 
%      M/S == Velocity in m/sec
%
%  To avoid inconsistencies, if units are not assigned an error occurs and the program terminates
%
      if strmatch(data.sensitivityUnits,'M/S')
          dtype='Velocity (m/s)';
          %trace = trace *1e4;
          data.data = data.data*1e3; % Convert from meters to mm
          dtype = 'Velocity';
          units = '(mm/s)';
      else
         disp(['ERROR[main]: units unkown.  Update code to accomodate ',data.sensitivityUnits])
         disp('ERROR[main]: Not able to create requested movie, now exiting')
         status_SeisSound = 0;
         break
      end
      metadata(irec).units= units;
      metadata(irec).dtype = dtype;


      if (debug)
         disp(['Data successfully loaded into structure database: ',num2str(irec),' recordeds read in.'])
      end
%
%  Now we have all the required data, next create the images, wav file, and movie for the data product
%
      if (debug)
         disp(['Making SeisSound data product for record number ',num2str(irec)]);
      end
      SeisSound_IRIS(metadata(irec), data, phaseTime,basedir)
%
%  Cycle completed for this seismogram, continue loop until all products have been created
%
      if (debug)
         disp(['Cycle complete for record: ',num2str(irec)])
      end
      irec = irec+1;
   end
end % end ista loop
%
%  Close input file and end
%
if fid>0
   fclose(fid);
end
